#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


## NACA 0012 airfoil configuration file for flow polar script
# Adrien Crovato

def getParam():
    p = {}
    # Input/Output
    p['File'] = '../models/n0012.geo' # Input file containing the model
    p['Pars'] = {'xLength' : 5, 'yLength' : 5, 'sEleFar' : 1.0, 'sEleAirfTE' : 0.01, 'sEleAirfLE' : 0.005} # Parameters for input file model
    p['Dim'] = 2 # Problem dimension
    p['Format'] = 'vtk' # Save format (vtk or gmsh)
    # Markers
    p['Fluid'] = 'field' # Name of physical group containing the fluid
    p['Farfield'] = ['upstream', 'farfield', 'downstream'] # LIST of name of physical groups containing the farfield boundaries (upstream/downstream should be first/last element)
    p['Wing'] = 'airfoil' # Name of physical group containing the airfoil boundary
    p['Wake'] = 'wake' # Name of physical group containing the wake
    p['Te'] =  'te' # Name of physical group containing the trailing edge
    # Freestream
    p['M_inf'] = 0.3 # Freestream Mach number
    p['AoA_begin'] = -3. # Freestream angle of attack (begin)
    p['AoA_end'] = 3. # Freestream angle of attack (end)
    p['AoA_step'] = 1. # Freestream angle of attack (step)
    # Geometry
    p['S_ref'] = 1. # Reference surface length (c_ref for 2D)
    p['c_ref'] = 1. # Reference chord length
    p['x_ref'] = 0.25 # Reference point for moment computation (x)
    p['y_ref'] = 0. # Reference point for moment computation (y)
    p['z_ref'] = 0. # Reference point for moment computation (z)
    # Numerical
    p['LSolver'] = 'SparseLU' # Linear solver (Pardiso, GMRES, MUMPS or SparseLU)
    p['NSolver'] = 'Picard' # Noninear solver type (Picard or Newton)
    p['Rel_tol'] = 1e-6 # Relative tolerance on solver residual
    p['Abs_tol'] = 1e-8 # Absolute tolerance on solver residual
    p['Max_it'] = 20 # Solver maximum number of iterations
    p['Relaxation'] = 0.7 # Relaxation parameter for Picard solver
    return p

def main():
    # Script call
    import flow.scripts.polar as p
    polar = p.Polar(getParam())
    polar.run()
    polar.disp()

if __name__ == "__main__":
    main()
