/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WADJOINT_H
#define WADJOINT_H

#include "flow.h"
#include "wObject.h"
#include "wTimers.h"

#include <iostream>
#include <vector>
#include <memory>
#include <Eigen/Sparse>

namespace flow
{

/**
 * @brief Adjoint solver class
 * @authors Adrien Crovato
 */
class FLOW_API Adjoint : public fwk::wSharedObject
{
public:
    std::shared_ptr<Newton> sol;                ///< Newton solver
    std::shared_ptr<tbox::MshDeform> morph;     ///< mesh morpher
    std::shared_ptr<tbox::LinearSolver> linsol; ///< linear solver

    int nthreads; ///< number of threads for the assembly
    int verbose;  ///< display more info

    std::vector<double> lamClFlo;        ///< lift flow adjoint
    std::vector<double> lamCdFlo;        ///< drag flow adjoint
    std::vector<Eigen::Vector3d> dClMsh; ///< lift gradient with respect to mesh
    std::vector<Eigen::Vector3d> dCdMsh; ///< drag gradient with respect to mesh
    double dClAoa;                       ///< lift gradient with respect to angle of attack
    double dCdAoa;                       ///< drag gradient with respect to angle of attack

private:
    fwk::Timers tms;                     ///< internal timers
    std::vector<std::vector<int>> arows; ///< rows (unknown index) with duplicated dof on wake

public:
    Adjoint(std::shared_ptr<Newton> _sol, std::shared_ptr<tbox::MshDeform> _morph, std::shared_ptr<tbox::LinearSolver> _linsol);
    virtual ~Adjoint() { std::cout << "~Adjoint()\n"; }

    void run();
    void save(tbox::MshExport *mshWriter, int n = 0);

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    // Adjoint solution and total gradients
    void solve();
    void computeGradientAoa();
    void computeGradientMesh();
    // Partial gradients wrt with respect to flow variables
    void buildGradientLoadsFlow(Eigen::VectorXd &dL, Eigen::VectorXd &dD);
    // Partial gradients with respect to angle of attack
    void buildGradientResidualAoa(Eigen::VectorXd &dR);
    void buildGradientLoadsAoa(double &dL, double &dD);
    // Partial gradients with respect to mesh
    void buildGradientResidualMesh(Eigen::SparseMatrix<double, Eigen::RowMajor> &dR);
    void buildGradientLoadsMesh(Eigen::RowVectorXd &dL, Eigen::RowVectorXd &dD);

    // @todo
    void mapRows();
};

} // namespace flow
#endif //WADJOINT_H
