/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wWake.h"
#include "wGroup.h"
#include "wFace.h"
#include <algorithm>
#include <unordered_set>
using namespace tbox;
using namespace flow;

Wake::Wake(std::shared_ptr<MshData> _msh, std::vector<int> const &nos)
    : Groups(_msh, nos)
{
    // Sanity check
    if (nos.size() != 3 && nos.size() != 4)
    {
        std::stringstream err;
        err << "flow::Wake should be built with 3 or 4 physical groups (for 2D or 3D resp.) but " << nos.size() << " were given!\n";
        throw std::runtime_error(err.str());
    }
    if (groups[0]->tag->elems.size() != groups[1]->tag->elems.size())
    {
        std::stringstream err;
        err << "flow::Wake: Groups " << nos[0] << " and " << nos[1] << " do not contain the same number of elements!\n";
        throw std::runtime_error(err.str());
    }
    // Build data structure
    connectNodes();
    createElements();
}
Wake::Wake(std::shared_ptr<MshData> _msh, std::vector<std::string> const &names)
    : Groups(_msh, names)
{
    // Sanity check
    if (names.size() != 3 && names.size() != 4)
    {
        std::stringstream err;
        err << "flow::Wake should be built with 3 or 4 physical groups (for 2D or 3D resp.) but " << names.size() << " were given!\n";
        throw std::runtime_error(err.str());
    }
    if (groups[0]->tag->elems.size() != groups[1]->tag->elems.size())
    {
        std::stringstream err;
        err << "flow::Wake: Groups " << names[0] << " and " << names[1] << " do not contain the same number of elements!\n";
        throw std::runtime_error(err.str());
    }
    // Build data structure
    connectNodes();
    createElements();
}
Wake::~Wake()
{
    for (size_t i = 0; i < wEle.size(); ++i)
    {
        delete wEle[i];
    }
    std::cout << "~Wake()\n";
}

/**
 * @brief Connect matching nodes on wake
 * @authors Adrien Crovato
 */
void Wake::connectNodes()
{
    // Temporary container for nodes
    std::vector<Node *> nodesUp, nodesLw;
    // Get unique node
    for (auto e : groups[0]->tag->elems)
        for (auto n : e->nodes)
            nodesUp.push_back(n);
    for (auto e : groups[1]->tag->elems)
        for (auto n : e->nodes)
            nodesLw.push_back(n);
    std::sort(nodesUp.begin(), nodesUp.end());
    auto it = std::unique(nodesUp.begin(), nodesUp.end());
    nodesUp.resize(std::distance(nodesUp.begin(), it));
    std::sort(nodesLw.begin(), nodesLw.end());
    it = std::unique(nodesLw.begin(), nodesLw.end());
    nodesLw.resize(std::distance(nodesLw.begin(), it));
    // Remove TE and/or tip nodes
    if (groups.size() == 4)
    {
        std::vector<Node *> edge;
        for (auto e : groups[3]->tag->elems)
            for (auto n : e->nodes)
                edge.push_back(n);
        std::sort(edge.begin(), edge.end());
        it = std::unique(edge.begin(), edge.end());
        edge.resize(std::distance(edge.begin(), it));
        // Find the intersection and remove
        std::vector<Node *>::iterator endRange;
        endRange = std::set_difference(nodesUp.begin(), nodesUp.end(),
                                       edge.begin(), edge.end(),
                                       nodesUp.begin());
        nodesUp.erase(endRange, nodesUp.end());
    }

    // Create the node map
    for (auto nUp : nodesUp)
    {
        //std::cout << "MASTER: " << nUp->pos(0) << ',' << nUp->pos(1) << ',' << nUp->pos(2) << std::endl;
        for (auto nLw : nodesLw)
        {
            //std::cout << "    SLAVE: " << nLw->pos(0) << ',' << nLw->pos(1) << ',' << nLw->pos(2) << std::endl;
            if ((nUp->pos - nLw->pos).norm() <= 1e-14)
            {
                nodMap[nUp] = nLw;
                break;
            }
        }
    }
    // Check if node map is not empty
    if (nodMap.size() != nodesUp.size())
    {
        std::stringstream err;
        err << "flow::Wake: some nodes from " << *groups[0]->tag << " could not be mapped to nodes from " << *groups[1]->tag << " (number of mapped nodes: " << nodMap.size() << ")!\n";
        throw std::runtime_error(err.str());
    }
}

/**
 * @brief Create Wake elements
 * @authors Adrien Crovato
 */
void Wake::createElements()
{
    // Temporary container
    std::map<Element *, Element *> suslE, suvuE, slvlE;
    std::map<Element *, std::vector<std::pair<size_t, Node *>>> asmbN;
    // Find the connected elements
    connectSurElements(suslE);
    connectVolElements(groups[0]->tag, groups[2]->tag, suvuE);
    connectVolElements(groups[1]->tag, groups[2]->tag, slvlE);
    // Find the node to be assembled
    findNodes(asmbN);

    // Create the Wake elements
    size_t i = 0;
    for (auto it = groups[0]->tag->elems.begin(); it != groups[0]->tag->elems.end(); it++, i++)
    {
        WakeElement *we = new WakeElement(i + 1, *it, suslE.at(*it), suvuE.at(*it), slvlE.at(suslE.at(*it)), asmbN.at(*it));
        wEle.push_back(we);
    }
}

/**
 * @brief Connect matching surface elements on upper/lower wake surface
 * @authors Adrien Crovato
 */
void Wake::connectSurElements(std::map<Element *, Element *> &ssMap)
{
    // Compare the position of each node of upper to lower surfaces
    for (auto uE : groups[0]->tag->elems)
    {
        for (auto lE : groups[1]->tag->elems)
        {
            size_t cnt = 0;
            for (auto uN : uE->nodes)
            {
                for (auto lN : lE->nodes)
                {
                    if ((uN->pos - lN->pos).norm() <= 1e-14)
                    {
                        cnt++;
                        break;
                    }
                }
                if (cnt == 0)
                    break;
            }
            // if all element nodes have the same position
            if (cnt == uE->nodes.size())
            {
                ssMap[uE] = lE;
                break;
            }
        }
    }
    // Check that all elements have been mapped
    if (ssMap.size() != groups[0]->tag->elems.size())
    {
        std::stringstream err;
        err << "flow::Wake: some elements from " << *groups[0]->tag << " could not be mapped to elements from " << *groups[1]->tag << "  (number of mapped elements: " << ssMap.size() << ")!\n";
        throw std::runtime_error(err.str());
    }
}

/**
 * @brief Connect matching volume elements over wake surface
 * @authors Adrien Crovato
 */
void Wake::connectVolElements(Tag *tagS, Tag *tagV, std::map<Element *, Element *> &svMap)
{
    // Create unordered set of wake faces
    std::unordered_set<Face *, EquFace, EquFace> wakFaces;
    for (auto e : tagS->elems)
    {
        Face *f = new Face(e->nodes);
        f->el0 = e;
        wakFaces.insert(f);
    }
    // Find volume faces that are wake faces
    for (auto e : tagV->elems)
    {
        size_t idx = 0;
        for (size_t i = 0; i < e->nodes.size(); ++i)
        {
            // build volume face
            std::vector<Node *> fN(e->nodes.size() - 1);
            for (size_t j = 0; j < e->nodes.size() - 1; ++j)
                fN[j] = e->nodes[(idx + j) % e->nodes.size()];
            Face f(fN);
            // check if face is a surface face
            // for some meshes, 1 tetra4 can have 2 faces that are surface tri3!
            auto it = wakFaces.find(&f);
            if (it != wakFaces.end())
                (*it)->el1 = e;
            idx++;
        }
    }
    // Create a map between the wake and volume elements
    for (auto f : wakFaces)
    {
        if (f->el1 == NULL)
        {
            std::stringstream err;
            err << "flow::Wake: no volume element from tag " << *tagV << "could be associated to surface element " << *(f->el0) << "!\n";
            throw std::runtime_error(err.str());
        }
        svMap[f->el0] = f->el1;
        delete f;
    }
}

/**
 * @brief Find nodes for which a contribution will be assembled
 * @authors Adrien Crovato
 */
void Wake::findNodes(std::map<Element *, std::vector<std::pair<size_t, Node *>>> &mapEN)
{
    // Check if nodes are in the node map and add them
    for (auto e : groups[0]->tag->elems)
    {
        std::vector<std::pair<size_t, Node *>> tmpN;
        for (size_t i = 0; i < e->nodes.size(); ++i)
            if (nodMap.find(e->nodes[i]) != nodMap.end())
                tmpN.push_back(std::pair<size_t, Node *>(i, e->nodes[i]));
        mapEN[e] = tmpN;
    }
}

void Wake::write(std::ostream &out) const
{
    out << "flow::Wake on " << *groups[0]->tag << " and " << *groups[1]->tag << "\n";
    for (auto p : nodMap)
        out << "\t" << p.first->no << " linked with " << p.second->no << "\n";
    for (auto we : wEle)
        we->write(out);
}
