#ifndef KATOPTRON_DIRICHLETLIST_H
#define KATOPTRON_DIRICHLETLIST_H

#include "katoptron.h"

#include "Map.h"
#include "wDirichlet.h"
#include "wProblem.h"
#include "wTag.h"
#include "wNode.h"

#include "ElementsList.h"

#include <Tpetra_Map.hpp>
#include <Tpetra_Vector.hpp>
#include <Teuchos_RCP.hpp>
#include <Kokkos_ViewFactory.hpp>

#include "EnsembleTraits.h"

namespace katoptron
{

/**
 * @brief Class which is used to store the list of the Dirichlet boundary conditions (BC).
 */
template <typename scalar>
class DirichletList
{
    typedef Map::local_ordinal_type local_ordinal_type;
    typedef Map::global_ordinal_type global_ordinal_type;

private:
    Kokkos::View<global_ordinal_type **, Kokkos::LayoutRight> dirichlet_nodesList;
    Kokkos::View<scalar ***, Kokkos::LayoutRight> dirichlet_values_list;

    local_ordinal_type dirichlet_number;
    local_ordinal_type dirichlet_size;
    local_ordinal_type dirichlet_nodes;

    local_ordinal_type dirichlet_time;
    local_ordinal_type dirichlet_dofs;
    local_ordinal_type dirichlet_values;

public:
    DirichletList(Problem &pbl, Teuchos::RCP<Map> map, Teuchos::RCP<ElementsList> elementsList);

    /**
     * @brief Return the number of Dirichlet BC.
     */
    inline local_ordinal_type getDirichletNumber() { return dirichlet_number; }

    /**
     * @brief Return the number of nodes for a
     * given Dirichlet BC.
     * 
     * Argument:
     * <ul>
     *  <li> i: the BC ID.
     * </ul>
     */
    inline local_ordinal_type getDirichletSize(local_ordinal_type i)
    {
        return dirichlet_nodesList(i, dirichlet_size);
    }

    /**
     * @brief Return the global ID of the node n1 of 
     * a given BC.
     * 
     * Arguments:
     * <ul>
     *  <li> i: the BC ID,
     *  <li> n1: the ID of the node on the current BC.
     * </ul>
     */
    inline global_ordinal_type getDirichletNode(local_ordinal_type i, local_ordinal_type n1)
    {
        return dirichlet_nodesList(i, dirichlet_nodes + n1);
    }

    /**
     * @brief Return whether the DOF j is fixed for a given BC.
     * 
     * Arguments:
     * <ul>
     *  <li> i: the BC ID,
     *  <li> j: the index of the DOF (0-3 if thermomechanical problems).
     * </ul> 
     */
    inline scalar getDirichletDof(local_ordinal_type i, local_ordinal_type j)
    {
        return dirichlet_values_list(i, 0, dirichlet_dofs + j);
    }

    /**
     * @brief Return the value of the fixed DOF for a given BC.
     * 
     * Arguments:
     * <ul>
     *  <li> i: the BC ID,
     *  <li> j: the index of the DOF (0-3 if thermomechanical problems).
     * </ul> 
     */
    inline scalar getDirichletValue(local_ordinal_type i, local_ordinal_type j)
    {
        return dirichlet_values_list(i, 0, dirichlet_values + j);
    }
};
}; // namespace katoptron

#endif //KATOPTRON_DIRICHLETLIST_H
