#ifndef KATOPTRON_LOADS_H
#define KATOPTRON_LOADS_H

#include "katoptron.h"
#include "Map.h"
#include "Domain.h"
#include "DirichletList.h"
#include "NeumannList.h"
#include "SourcesList.h"
#include "ElementsList.h"
#include "PreloadList.h"
#include "WeightsList.h"

namespace katoptron
{

/**
 * @brief Class which includes all the loads, boundary conditions, preloads, and weights.
 */
template <typename scalar>
class Loads
{
public:
    Teuchos::RCP<DirichletList<scalar>> dirichletList;
    Teuchos::RCP<NeumannList<scalar>> neumannList;
    Teuchos::RCP<SourcesList<scalar>> sourcesList;
    Teuchos::RCP<PreloadList<scalar>> preloadList;
    Teuchos::RCP<WeightsList<scalar>> weightsList;

    Loads();
    Loads(Problem &pbl, Teuchos::RCP<Map> map, Teuchos::RCP<Domain<scalar>> domain);
};

/**
 * @brief Loads constructor
 * 
 * This constructor initializes the lists of Dirichlet BC, Neumann BC, sources, preloads, and weights.
 * 
 * Arguments:
 * <ul>
 *  <li> pbl: A Problem object,
 *  <li> map: an RCP to a Map object,
 *  <li> domain: an RCP to a Domain object.
 * </ul>
 */
template <typename scalar>
Loads<scalar>::Loads(Problem &pbl, Teuchos::RCP<Map> map, Teuchos::RCP<Domain<scalar>> domain)
{
    dirichletList = Teuchos::rcp(new DirichletList<scalar>(pbl, map, domain->elementsList));
    neumannList = Teuchos::rcp(new NeumannList<scalar>(pbl, map, domain->elementsList));
    sourcesList = Teuchos::rcp(new SourcesList<scalar>(pbl, map, domain->elementsList));
    preloadList = Teuchos::rcp(new PreloadList<scalar>(pbl, map, domain->elementsList));
    weightsList = Teuchos::rcp(new WeightsList<scalar>(pbl, map, domain->elementsList));
}

}; // namespace katoptron

#endif //KATOPTRON_LOADS_H
