#ifndef KATOPTRON_PRELOADLIST_H
#define KATOPTRON_PRELOADLIST_H

#include "katoptron.h"

#include "Map.h"
#include "wContact.h"
#include "wTag.h"

#include <Tpetra_Map.hpp>
#include <Tpetra_Vector.hpp>
#include <Teuchos_RCP.hpp>
#include <Kokkos_ViewFactory.hpp>

#include "EnsembleTraits.h"

namespace katoptron
{

/**
 * @brief Class which is used to store the list of the preloads.
 */
template <typename scalar>
class PreloadList
{
    typedef Map::local_ordinal_type local_ordinal_type;
    typedef Map::global_ordinal_type global_ordinal_type;

private:
    Kokkos::View<scalar *, Kokkos::LayoutRight> contacts_dg;

public:
    PreloadList(Problem &pbl, Teuchos::RCP<Map> map, Teuchos::RCP<ElementsList> elementsList);

    /**
     * @brief Return the value of the preload for
     * a given contact interface.
     * 
     * Argument:
     * <ul>
     *  <li> i: the contact interface ID.
     * </ul> 
     */
    inline scalar getPreloadValue(local_ordinal_type i)
    {
        return contacts_dg(i);
    }
};

/**
 * @brief NeumannList constructor
 * 
 * This constructor allocates 1 Kokkos::View which stores all the preload related to each contact interface.
 * 
 * Arguments:
 * <ul>
 *  <li> pbl: a Problem object,
 *  <li> map: an RCP to a Map object,
 *  <li> elementsList: an RCP to an ElementList object.
 * </ul>
 */
template <typename scalar>
PreloadList<scalar>::PreloadList(Problem &pbl, Teuchos::RCP<Map> map, Teuchos::RCP<ElementsList> elementsList)
{
    int contacts_number = pbl.Contacts.size();

    typedef EnsembleTraits<scalar> ET;
    const int ensemble_size = ET::size;

    contacts_dg = Kokkos::View<scalar *, Kokkos::LayoutRight>("R", contacts_number);
    for (auto i = 0; i < contacts_number; ++i)
        for (int j = 0; j < ensemble_size; ++j)
            ET::coeff(contacts_dg(i), j) = pbl.Contacts[i]->dg_vector[j];
}
}; // namespace katoptron

#endif //KATOPTRON_PRELOADLIST_H
