#!/usr/bin/env python3
# -*- coding: utf-8; -*-

import threading
import katoptron as m
import tbox
import tbox.gmsh as gmsh
import katoptron.utilities as u
from PyTrilinos import Teuchos
import shlex
import subprocess
import os
import numpy as np
import tarfile
import glob
from fwk.wutils import parseargs


def model(msh,
          E,
          p,
          ensemble_size,
          case_name,
          work_dir,
          max_iter,
          comm,
          use_Prec=False):
    rank = comm.rank
    siz = comm.size
    file_dir = os.path.dirname(__file__)

    pbl = m.Problem(msh, comm)

    nu = 0.2 * np.ones(ensemble_size)
    k = 0.114
    d = 0.

    zero = np.zeros(ensemble_size)
    m.Medium(pbl, "Body 1", "test", E, nu, k, d)

    m.Dirichlet(pbl, "Surf 1 1", "Clamped", 1, 0., 1, 0., 1, 0., 1, 0.,
                ensemble_size)
    m.Neumann(pbl, "Surf 1 2", "heat fluxes", 0, zero, 0, zero, 1, p, 0, zero,
              ensemble_size)

    solverList = Teuchos.ParameterList()
    solverList['Ensemble Convergence Tolerance'] = 0.  # 10**(-8)
    solverList['Maximum Iterations'] = max_iter
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    solverList['Use preconditioner'] = use_Prec

    solverList['type'] = "BlockGmres"
    solverList['Maximum active set iteration'] = 1

    solverList['Print Teuchos timers'] = True
    solverList['Write vtk files'] = False

    if use_Prec:
        solverList['convert MueLu xml file'] = True
        solverList["use xml file"] = True
        solverList["MueLu xml file name"] = file_dir + '/prec.xml'

    slv = m.IterativeSolver(pbl, solverList, 3, ensemble_size)

    if not os.path.exists(case_name):
        os.makedirs(case_name)
    os.chdir(case_name)

    if rank == 0:
        f = open('msh_size.txt', 'w')
        print(len(msh.nodes), file=f)
        f.closed

        np.savetxt('E.txt', E)
        np.savetxt('p.txt', p)

    slv.start()

    if rank == 0:
        f = open('timers.txt', 'w')
        print(slv.getTimers(), file=f)
        f.closed

    os.chdir(work_dir)


def one_model(test_id, ensemble_size, rank, work_dir, msh, E, p, k, max_iter,
              comm):
    directory_case_id = work_dir + '/test_' + str(test_id) + '_s_' + str(
        ensemble_size) + '_rank_' + str(rank)
    model(msh,
          E,
          p,
          ensemble_size,
          directory_case_id,
          work_dir,
          k,
          max_iter,
          comm,
          use_Prec=True)


def main():
    comm, rank, size = m.utilities.import_MPI()

    group_size = 1
    color = int(np.floor((rank / group_size)))
    key = rank

    newcomm = comm.Split(color, key)

    args = parseargs()
    work_dir = os.getcwd()

    msh_name = '1_cube.msh'

    E_mean = 1.
    p_mean = 0.1

    ensemble_size = int(os.getenv('ensemble_size', 1))
    test_id = int(os.getenv('test_id', 1))
    max_iter = int(os.getenv('max_iter', 100))

    msh = gmsh.MeshLoader(msh_name, work_dir).execute()

    E = E_mean * np.ones(ensemble_size)
    p = p_mean * np.ones(ensemble_size)

    directory_case_id = 'test_' + str(test_id) + '_s_' + str(
        ensemble_size) + '_rank_' + str(rank)

    nThreads = u.Initialize_Kokkos()
    model(msh,
          E,
          p,
          ensemble_size,
          directory_case_id,
          work_dir,
          max_iter,
          newcomm,
          use_Prec=True)

    u.Finalize_Kokkos()


if __name__ == "__main__":
    main()
